#ifndef _EMISSIVE_SAMPLE_SET_
#define _EMISSIVE_SAMPLE_SET_

#include <Inventor/nodes/SoShape.h>
#include <Inventor/fields/SoMFVec3f.h>
#include <Inventor/fields/SoMFColor.h>
#include <Inventor/fields/SoMFFloat.h>
#include <Inventor/fields/SoSFFloat.h>
#include <Inventor/fields/SoSFBool.h>
#include <GL/gl.h>

//////////////////////////////////////////////////////////////////////////////
//
//  Class: EmissiveSampleSet
//
//  Efficiently renders disc shaped samples with radius 'sampleRadius' at
//  position 'location' with orientation 'normal' and color 'color'.
//
//  The purpose of creating this class was to efficiently render thousands of
//  samples with varying colors and transformations.  This sort of task
//  cannot be accomplished with standard Open Inventor nodes, OpenGL must be
//  exploited.  The main feature of this class is that the samples are
//  rendered with GL_EMISSION not the standard GL_DIFFUSE.
//
//////////////////////////////////////////////////////////////////////////////

class EmissiveSampleSet : public SoShape {

  // Required by Inventor for creating a node
  SO_NODE_HEADER(EmissiveSampleSet);

public:

  // the radius of each sample being drawn.
  //SoSFFloat    sampleRadius;
  SoSFBool     touch;

  // the following are the fields used to store all the information needed
  // for each sample.

  // The location of each sample in world coordinates
  SoMFVec3f    location;
  // The normal direction the sample is to face
  SoMFVec3f    normal;
  // The color of the sample
  SoMFColor    color;
  // The radius of the sample
  SoMFFloat    radius;
  // The radius of the sample
  SoSFFloat    singleRadius;

  // Required by Inventor so it can identify nodes of this type
  static void initClass ();

  // default constructor where fields are initalized and created.
  EmissiveSampleSet ();

protected:
  // All the OpenGL calls are made here
  virtual void GLRender (SoGLRenderAction *action);

  // Output geometric primitives as described by SoShape
  virtual void generatePrimitives (SoAction *action);

  // Compute the Bounding box of the node
  virtual void computeBBox (SoAction *action,
			    SbBox3f &box, SbVec3f &ctr);

private:

  virtual ~EmissiveSampleSet ();

};

#endif /* _EMISSIVE_SAMPLE_SET_ */
